package Telephony::CTPort;

# CTPort - part of ctserver client/server library for Computer Telephony 
# programming in Perl
#
# Copyright (C) 2001-2003 David Rowe
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
=pod
=head1 NAME

Telephony::CTPort - Computer Telephony programming in Perl

=head1 SYNOPSIS

use Telephony::CTPort;

$ctport = new Telephony::CTPort(1200); # first port of CT card
$ctport->off_hook;
$ctport->play("beep");                 
$ctport->record("prompt.wav",5,"");    # record for 5 seconds
$ctport->play("prompt.wav");           # play back
$ctport->on_hook;

=head1 DESCRIPTION

This module implements an Object-Oriented interface to control Computer 
Telephony (CT) card ports using Perl.  It is part of a client/server
library for rapid CT application development using Perl.

=head1 AUTHOR

David Rowe, support@voicetronix.com.au

Ben Kramer, support@voicetronix.com.au

=cut

require 5.005_62;
#use strict;
use warnings;
use Carp;
use IO::Socket;
use IO::Handle;
use Cwd qw(cwd);
use POSIX;
use Fcntl ':flock';

require Exporter;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use Telephony::CTPort ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw( ) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw( );
our $VERSION = '1.01';

# Preloaded methods go here.

# constructor - opens TCP/IP connection to server and makes sure we are
# on hook to start with
=pod

=head1 CONSTRUCTOR

new Telephony::CTPort(SERVER_PORT);

Connects Perl client to the "ctserver" server via TCP/IP port SERVER_PORT,
where SERVER_PORT=1200, 1201,..... etc for the first, second,..... etc
CT ports.

=head1 METHODS

=cut
sub new($) {
	my $proto = shift;
	our $port = shift;
	my $class = ref($proto) || $proto;
	my $self = {};

	$self->{SERVER} = IO::Socket::INET->new(
		Proto => "tcp",
		PeerAddr => "localhost",
		PeerPort => $port,
	)
	or croak "cannot connect to server tcp port $port";

	$self->{EVENT}  = undef;		# holds the events
	$self->{DEF_EXT} = ".au";		# default audio file extension
	$self->{PATHS} = [];			# user supplied audio file paths
	$self->{INTER_DIGIT} = undef;		# holds the IDD
	$self->{DEFEVENTS} = undef;		# Used to hold default event handlers
	$self->{CONFIG} = undef;		# Used to hold Config values
	$self->{DAEMON} = undef;		# Used to hold daemon state
	$self->{HANDLE}= $port-1200;		# our handle number
	$self->{IGNORE_DTMF} = 0;		# holds the ignore dtmf state

	bless($self, $class);
	return $self;
}

=pod

=head2 set_def_ext()

=over

Used to set the default extension of audio files. The default is ".au". It can also
be overridden by giving the full name including extension of the file.

 $ctport->set_def_ext(".wav");

=back

=cut
sub set_def_ext($) {
	my $self = shift;
	my $defext = shift;
	$self->{DEF_EXT} = $defext;
}

=pod

=head2 set_paths()

=over

Used to set the search paths for file playback.

 $ctport->set_paths($moh_path);

=back

=cut
sub set_paths($) {
	my $self = shift;
	my $paths = shift;
	$self->{PATHS} = $paths;
}

=pod

=head2 set_event_handler()

=over

Sets a subroutine to be called when a certain event occurs.

 $ctport->set_event_handler("incoming call","port_busy");

=back

=cut

sub set_event_handler(){
	my $self = shift;
	my $event = shift;
	my $handle = shift;
	$self->{DEFEVENT}{$event}=$handle;
}

=pod

=head2 unset_event_handler()

=over

Unsets a handler.

 $ctport->unset_event_handler("incoming call");

=back

=cut

sub unset_event_handler(){
	my $self = shift;
	my $event = shift;
	$self->{DEFEVENT}{$event}=undef;
}

=pod 

=head2 event()

=over

Returns the most recent event, or undef if no events pending.

 $event=$ctport->event();

=back

=cut

sub event($) {
	my $self = shift;
	return $self->{EVENT};
}

=pod

=head2 off_hook()

=over

Takes port off hook, just like picking up the phone.

 $ctport->off_hook();

=back

=cut

sub off_hook() {
	my $self = shift;
	my $buf;
	my $server = $self->{SERVER};
	$self->send_soc("cmd=ctanswer src=".$self->{HANDLE}." dst=".$self->{HANDLE}." argc=0 \n");
	$buf =$self->get_reply($server);
}

=pod

=head2 on_hook()

=over

Places the port on hook, just like hanging up.

 $ctport->on_hook();

=back

=cut

sub on_hook() {
	my $self = shift;
	my $buf;
	my $server = $self->{SERVER};
	$self->send_soc("cmd=cthangup src=".$self->{HANDLE}." dst=".$self->{HANDLE}." argc=0 \n");
	$buf =$self->get_reply($server);
}

=pod

=head2 wait_for_ring()

=over

Blocks until port detects 2 rings (by default), then returns. 
You can pass it a number of rings to wait. The caller ID (if 
present) will be returned.

 $cid=$ctport->wait_for_ring();

 $cid=$ctport->wait_for_ring(3);

=back

=cut

sub wait_for_ring() {
	my $self = shift;
	my $num_ring = shift;
	if (!defined $num_ring){
		$num_ring=2;
	}
	my $server = $self->{SERVER};
	my $caller_id;
	$self->send_soc("cmd=ctwaitforring src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=$num_ring argc=1 \n");
	my $result=$self->get_reply($server);
	$caller_id = get_value($result,"arg2");
	return $caller_id;
}

=pod

=head2 wait_for_dial_tone()

=over

Blocks until dial tone detected on port, then returns.

 $ctport->wait_for_dial_tone();

=back

=cut

sub wait_for_dial_tone() {
	my $self = shift;
	my $server = $self->{SERVER};
	my $buf;
	$self->send_soc("cmd=ctwaitfordial src=".$self->{HANDLE}." dst=".$self->{HANDLE}." argc=0 \n");
	$buf =$self->get_reply($server);
}

=pod

=head2 play_busy_tone_async()

=over

Starts the port playing a busy tone.

 $ctport->play_busy_tone_async();

=back

=cut

sub play_busy_tone_async(){
	my $self = shift;
	my $server = $self->{SERVER};
	$self->send_soc("cmd=ctplaytoneasync src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=busy argc=1 \n");
	my $tmp=$self->get_reply($server);
	$tmp = get_value($tmp,"arg1");
	return($tmp);
}

=pod

=head2 play_dialx_tone_async()

=over

Starts the port playing an external dial tone.

 $ctport->play_dialx_tone_async();

=back

=cut

sub play_dialx_tone_async(){
	my $self = shift;
	my $server = $self->{SERVER};
	$self->send_soc("cmd=ctplaytoneasync src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=dialx argc=1 \n");
	my $tmp=$self->get_reply($server);
	$tmp = get_value($tmp,"arg1");
	return($tmp);
}

=pod

=head2 play_dial_tone_async()

=over

Starts the port playing an dial tone.

 $ctport->play_dial_tone_async();

=back

=cut

sub play_dial_tone_async(){
	my $self = shift;
	my $server = $self->{SERVER};
	$self->send_soc("cmd=ctplaytoneasync src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=dial argc=1 \n");
	my $tmp=$self->get_reply($server);
	$tmp = get_value($tmp,"arg1");
	return($tmp);
}

=pod

=head2 play_ringback_async()

=over

Starts the port playing ringback tone.

 $ctport->play_ringback_async();

=back

=cut

sub play_ringback_async(){
	my $self = shift;
	my $server = $self->{SERVER};
	$self->send_soc("cmd=ctplaytoneasync src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=ringback argc=1 \n");
	my $tmp=$self->get_reply($server);
	$tmp = get_value($tmp,"arg1");
	return($tmp);
}

=pod

=head2 play_terminate()

=over

Stops the port playing an asyncronous tone.

 $ctport->play_terminate();

=back

=cut

sub play_terminate(){
	my $self = shift;
	my $server = $self->{SERVER};
	$self->send_soc("cmd=ctstoptone src=".$self->{HANDLE}." dst=".$self->{HANDLE}." argc=0 \n");
	my $tmp=$self->get_reply($server);
	$tmp = get_value($tmp,"arg1");
	return($tmp);
}

=pod

=head2 play_stop()

=over

Stops the port playing an asyncronous file.

 $ctport->play_stop();

=back

=cut

sub play_stop(){
	my $self = shift;
	my $server = $self->{SERVER};
	$self->send_soc("cmd=ctplay_stop src=".$self->{HANDLE}." dst=".$self->{HANDLE}." argc=0 \n");
	my $tmp=$self->get_reply($server);
	$tmp = get_value($tmp,"arg1");
	return($tmp);
}

=pod

=head2  play_async()

=over

Starts the port playing an asyncronous file(s).

 $ctport->play_async("hello.wav welcome.wav");

=back

=cut

sub play_async($) {
	my $self = shift;
	my $files_str = shift;
	my $file;

	unless (length($files_str)) {return;}

	foreach $file (split(/ /,$files_str)){
		$self->_ctplayonefile_async($file);
	}
}

sub _ctplayonefile_async() {
	my $self = shift;
	my $file = shift;
	my $server = $self->{SERVER};
	my $path;
	my $result;

	# append default extension if no extension on file name
	if ($self->{DEF_EXT}) {
		if ($file !~ /\./) {
			$file = $file . $self->{DEF_EXT};
		}
	}

	if (defined($self->{PATHS})) { # check user supplied paths
		my @paths = $self->{PATHS};
		foreach $path (@paths) {
			# find first path that contains the file
			if (-e "$path/$file") {
				$self->send_soc("cmd=ctplay_async src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=$path/$file argc=1 \n");
				$result =$self->get_reply($server);
				return;			      
			}
		}
	}
        if (-e "$ENV{PWD}/$file") { # check default paths
		$self->send_soc("cmd=ctplay_async src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=$ENV{PWD}/$file argc=1 \n");
		$result =$self->get_reply($server);
		return;			      
	}
	elsif (-e "$ENV{PWD}/prompts/$file") {
		# prompts sub-dir of current dir
		$self->send_soc("cmd=ctplay_async src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=$ENV{PWD}/prompts/$file argc=1 \n");
		$result =$self->get_reply($server);
		return;			      
	}
	elsif (-e "/usr/share/vpb/USEngM/$file") { # USEngM prompts dir
		$self->send_soc("cmd=ctplay_async src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=/usr/share/vpb/USEngM/$file argc=1 \n");
		$result =$self->get_reply($server);
		return;			      
	}
	elsif (-e "$file") { # Full path supplied
		$self->send_soc("cmd=ctplay_async src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=$file argc=1 \n");
		$result =$self->get_reply($server);
		return;			      
	}

	$self->logger("play: File [$file] not found!");
	return -1;
} 

=pod

=head2 play()

=over

Plays an audio file(s).

 $ctport->play("hello.wav");

Playing stops immediately if a DTMF key is 
pressed.  The DTMF key pressed can be read using the event() member function.
If $ctport->event() is already defined it returns immediately.  Any digits
pressed while playing will be added to the digit buffer.

Filename extensions:

=over 4

=item *

default is .au, can be redefined by calling set_def_ext()

=item *

override default by providing extension, e.g. $ctport->play("hello.wav");

=back

Searches for file in:

=over 4

=item *

paths defined by set_path() method

=item *

current dir

=item *

"prompts" sub dir (relative to current dir)

=item *

full path supplied by caller

=item *

/usr/share/vpb/USEngM

=back

You can play multiple files, e.g. 

$ctport->play("Hello World"); 

(assumes you have Hello.au and World.au files available)

You can "speak" a limited vocab, e.g. 

$ctport->play("1 2 3"); 

(see /usr/share/vpb/USEngM directory for the list of included files that define
the vocab)


=back

=cut

sub play($) {
	my $self = shift;
	my $files_str = shift;
	my $file;
	my $result="";

	unless (length($files_str)) {return;}
	my @files_array = split(/ /,$files_str);

	foreach $file (@files_array) {
		if (!$self->{EVENT}) {
			$self->logger("Playing $file");
			$result=$self->_ctplayonefile($file);
			if($result eq "DTMF"){
				$self->logger("Play got DTMF");
				return "DTMF";
			}
		}
		else {
			$self->logger("Event in queue not playing $file");
		}
	}
	if (!$self->{EVENT}) {
		return "OK";
	}
	else {
		return "EVENT";
	}
}

sub _ctplayonefile() {
	my $self = shift;
	my $file = shift;
	my $server = $self->{SERVER};
	my $event;
	my $path;
	my $result="";
	my $playing=0;

	# append default extension if no extension on file name
	if ($self->{DEF_EXT}) {
		if ($file !~ /\./) {
			$file = $file . $self->{DEF_EXT};
		}
	}

	# check user supplied paths
	if (defined($self->{PATHS})) {
		my @paths = $self->{PATHS};
		foreach $path (@paths) {
			# find first path that contains the file
			if (-e "$path/$file") {
				if ($self->{IGNORE_DTMF} == 0){
					$self->send_soc("cmd=ctplay src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=$path/$file argc=1 \n");
				}
				else {
					$self->send_soc("cmd=ctplay src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=$path/$file arg2=ignore_dtmf argc=2 \n");
				}
				$playing=1;
			}
		}
	}
	if (-e "$ENV{PWD}/$file") {
		# full path supplied by caller
		if ($self->{IGNORE_DTMF} == 0){
		$self->send_soc("cmd=ctplay src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=".$ENV{PWD}."/$file argc=1 \n");
		}
		else {
			$self->send_soc("cmd=ctplay src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=".$ENV{PWD}."/$file arg2=ignore_dtmf argc=2 \n");
		}
		$playing=1;
	}
	elsif (-e "$ENV{PWD}/prompts/$file") {
		# prompts sub-dir of current dir
		if ($self->{IGNORE_DTMF} == 0){
			$self->send_soc("cmd=ctplay src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=".$ENV{PWD}."/prompts/$file argc=1 \n");
		}
		else {
			$self->send_soc("cmd=ctplay src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=".$ENV{PWD}."/prompts/$file arg2=ignore_dtmf argc=2 \n");
		}
		$playing=1;
	}
	elsif (-e "/usr/share/vpb/USEngM/$file") {
		# USEngM prompts dir
		if ($self->{IGNORE_DTMF} == 0){
			$self->send_soc("cmd=ctplay src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=/usr/share/vpb/USEngM/$file argc=1 \n");
		}
		else {
			$self->send_soc("cmd=ctplay src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=/usr/share/vpb/USEngM/$file arg2=ignore_dtmf argc=2 \n");
		}
		$playing=1;
	}
	elsif (-e "$file") {
		# Full path supplied
		if ($self->{IGNORE_DTMF} == 0){
			$self->send_soc("cmd=ctplay src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=$file argc=1 \n");
		}
		else {
			$self->send_soc("cmd=ctplay src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=$file arg2=ignore_dtmf argc=2 \n");
		}
		$playing=1;
	}
	if(!$playing){
		$self->logger("play: File [$file] not found!");
		return "ERROR";
	}
	else {
		$result =$self->get_reply($server);
		$event = get_value($result,"arg1");
		if ($event eq "EVENT"){
			$event = get_value($result,"arg3");
			$self->{EVENT} = $event;
			return "EVENT";			      
		}
		elsif ($event eq "DTMF"){
			$event = get_value($result,"arg2");
			$self->{EVENT} = $event;
			return "DTMF";			      
		}
		return "OK";			      
	}
	return "ERROR";
}

=pod

=head2 record()

=over

Records to a file, terminating on time, digits, an event or silence timeout.

 $ctport->record($file_name,$time_out,$term_keys,$silence_timeout);

If silence_timeout is not defined or zero, it defaults to time_out.
Records $file_name for $time_out seconds or until any of the digits in 
$term_keys are pressed.  The path of $file_name is considered absolute if 
there is a leading /, otherwise it is relative to the current directory.


=back

=cut

sub record($$$) {
	my $self = shift;
	my $file = shift;
	my $timeout = shift;
	my $term_digits = shift;
	my $silence_timeout = shift;
	my $server = $self->{SERVER};
	my $event;
	my $result;
	my @unpacked_file = split(//, $file);

	if ($self->{EVENT}) {
		$self->logger("Event in queue not recording $file");
		return;
	}

	unless ($unpacked_file[0] eq "/") {
		# if not full path, record in current dir
		$file = "$ENV{PWD}/$file";
	}
	if (!defined $silence_timeout){
		$silence_timeout=0;
	}
	$self->send_soc("cmd=ctrecord src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=$file arg2=$timeout arg3=$term_digits arg4=$silence_timeout argc=4 \n");
	$result =$self->get_reply($server);
	$event = get_value($result,"arg1");
	if ($event eq "EVENT"){
		$event = get_value($result,"arg3");
		#$event =~ s/[^0-9ABCD#*]//g;
		$self->{EVENT} = $event;
	}
	return;
}

=pod

=head2  ctsleep() 

=over

Blocks for $seconds, unless a DTMF key is pressed in which
case it returns immediately.  If $ctport->event() is already defined it 
returns immediately without sleeping.

 $ctport->ctsleep($seconds);


=back

=cut

sub ctsleep($) {
	my $self = shift;
	my $secs = shift;
	my $server = $self->{SERVER};
	my ($argc,$event);

	if (!defined($self->{EVENT})) {
		$self->send_soc("cmd=ctsleep src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=$secs argc=1 \n");
		$result =$self->get_reply($server);
		$argc = get_value($result,"argc");
		if ($argc eq "2"){
			$event = get_value($result,"arg2");
			#$event =~ s/[^0-9ABCD#*]//g;
			$self->{EVENT} = $event;
		}
		if ($argc eq "3") {
			$self->{EVENT} = get_value($result,"arg3");
		}

	}
}

=pod

=head2 clear()

=over

Clears any pending events, and clears the DTMF digit buffer.

 $ctport->clear();

=back

=cut

sub clear() {
	my $self = shift;
	my $server = $self->{SERVER};
	my $tmp;
	$self->send_soc("cmd=ctclear src=".$self->{HANDLE}." dst=".$self->{HANDLE}." argc=0 \n");
	$tmp =$self->get_reply($server); 
	if (defined $self->{EVENT}){
		$self->logger("WARNING: clearing event [".$self->{EVENT}."]");
	}
	undef $self->{EVENT};
	return;
}

=pod

=head2 collect()

=over

 $digits=$ctport->collect($max_digits,$max_seconds,$IDD);

Returns up to $max_digits by waiting up to $max_seconds.  Will return as soon
 as either $max_digits have been collected or $max_seconds have elapsed.

DTMF digits pressed at any time are collected in the digit buffer.  The digit
buffer is cleared by the clear() method.  Thus it is possible for this function
to return immediately if there are already $max_digits in the digit buffer.

=back

=cut

sub collect($$) {
	my $self = shift;
	my $maxdigits = shift;
	my $maxseconds = shift;
	my $maxinter;
	my $server = $self->{SERVER};
	my $result ="OK";
	my $digits ;

	$maxinter = $self->{INTER_DIGIT} || $maxseconds;
	undef $self->{EVENT};

	$self->send_soc("cmd=ctcollect src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=$maxdigits arg2=$maxseconds arg3=$maxinter argc=3 \n");
	while ($result eq "OK"){
		$result =$self->get_reply($server); 
	}
	$digits=get_value($result,"arg1");
	if ($digits eq "OK"){
		$digits=get_value($result,"arg2");
	}
	if ($digits eq "EVENT"){
		$event = get_value($result,"arg3");
		$self->{EVENT} = $event;
	}
	$self->logger("collected: $digits");
	return $digits;		  
}

=pod

=head2  dial()

=over

Dials a DTMF string.  Valid characters are 1234567890#*,&ABCD

 $result=$ctport->dial("0,5555555");

=over 4

=item *

, gives a 1 second pause, e.g. $ctport->dial(",,1234) will wait 2 seconds, 
then dial extension 1234.

=item *

& generates a hook flash (used for transfers on many PBXs) e.g. :

$ctport->dial("&,1234) will send a flash, wait one second, then dial 1234. 

=back

=back

=cut

sub dial($) {
	my $self = shift;
	my($dial_str) = shift;
	my $server = $self->{SERVER};
	my($tmp);
	$self->send_soc("cmd=ctdial src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=$dial_str argc=1 \n");
	$tmp =$self->get_reply($server); 
	return($tmp);
}

=pod

=head2 number()

=over

Returns a string of audio files that enable numbers to be "spoken"

 $playfiles=$ctport->number("123456");

e.g. number() will convert 121 into "one hundred twenty one" 

e.g. ctplay("youhave " . $ctnumber($num_mails) . " mails");

(assumes files youhave.au, mails.au, and variable $num_mails exist)



=back

=cut

sub number($) {
	my $self = shift;
	my $num = shift;
	my $server = $self->{SERVER};
	my $tens;
	my $hundreds;
	my @files;
	my $all;
	unless ($num) {return undef};
	if ($num == 0) { push(@files, $num); }

	$hundreds = int($num/100) * 100;
	$num = $num - $hundreds;
	if ($hundreds != 0) { push(@files, $hundreds); }
	$tens = int($num/10) * 10;
	if ($num > 20) { 
		$num = $num - $tens;
		if ($tens != 0) { push(@files, $tens); }
	}
	if ($num != 0) { push(@files, $num); }
	$all = "@files";
	return $all;
}

=pod

=head2 get_inter_digit_time_out()

=over

Will return the Inter-Digit-Delay;

 $IDD=$ctport->get_inter_digit_time_out();

=back

=cut

sub get_inter_digit_time_out() {
	my $self = shift;
	return $self->{INTER_DIGIT};
}

=pod

=head2 set_inter_digit_time_out()

=over

Will set the Inter-Digit-Delay;

 $ctport->set_inter_digit_time_out($IDD);

=back

=cut

sub set_inter_digit_time_out($) {
	my $self = shift;
	my $inter = shift;
	$self->{INTER_DIGIT} = $inter;
}

=pod

=head2 wait_for_event()

=over

Waits for an event to occur, and returns the handle for the port that caused the event and the event.

 ($handle,$event)=$ctport->wait_for_event();

=back

=cut

sub wait_for_event() {
	my $self = shift;
	my $server = $self->{SERVER};
	my $result;
	my $handle = "";
	my ($tmp,$i);
	my $event="";
	my @args;

	while(1){
		$self->send_soc("cmd=ctwaitforevent src=".$self->{HANDLE}." dst=".$self->{HANDLE}." argc=0 \n");
		$result =$self->get_reply($server);
		$tmp =get_value("$result","arg1");
		$event = get_value($result,"arg2");
		$handle = get_value($result,"arg3");
		if (($tmp eq "OK")&&($event ne "")&&($handle ne "")){
			if($event eq "message"){
				#$self->logger("checking for message");
				$self->send_soc("cmd=ctgetmessage src=".$self->{HANDLE}." dst=".$self->{HANDLE}." argc=0 \n");
				$result =$self->get_reply($server);
				#$self->logger("Got: $result");
				$handle=get_value($result,"dst");
				$event = get_value($result,"arg1");
				$argc = get_value($result,"argc");
				for ($i=2; $i<=$argc; $i++){
					$tmp=get_value($result,"arg$i");
					push @args,$tmp;
				}
			}
			if (defined $self->{DEFEVENT}{$event}){
				#my $foo="main::".$self->{DEFEVENT}{$event};
				my $foo=$self->{DEFEVENT}{$event};

				print "event $event calling $foo\n";
				
				#&{$foo}($handle);
				&$foo($handle);
				$self->{EVENT} = $event;
				undef $event;
				$handle="";
			}
			else {
				# only return if not handled by default
				$self->{EVENT} = $event;
				return($handle,$event,@args);
			}
		}
	}
}

=pod

=head2  send_event()

=over

Sends an event to another port.

 $ctport->send_event($other_port,"goodbye");

=back

=cut

sub send_event($$) {
	my $self = shift;
	my $port = shift;
	my $event = shift;
	my $server = $self->{SERVER};
	my $tmp;

	$self->send_soc("cmd=ctsendevent src=".$self->{HANDLE}." dst=$port arg1=$event argc=1 \n");
	$tmp =$self->get_reply($server); 
	return;
}

=pod

=head2  send_message()

=over

This extends the capabilites of send_event.
 
 @message=("clicktodial","1","84155104");
 $ctport->send_message($other_port,@message);

=back

=cut

sub send_message($@) {
	my $self = shift;
	my $port = shift;
	my $server = $self->{SERVER};
	my $tmp;
	my $args="";
	my $argc=0;
	my $bit;
	while (defined($bit=shift)){
		$argc++;
		$args.=" arg".$argc."=".$bit;
	}
	$self->send_soc("cmd=ctsendevent src=".$self->{HANDLE}." dst=$port".$args." argc=$argc \n");
	$tmp =$self->get_reply($server); 
	return;
}

=pod

=head2 terminating play(s) start_timer_async()

=over

Starts a timer, returning immediatly.

 $result=$ctport->start_timer_async($duration);

=back

=cut

sub start_timer_async($) {
	my $self = shift;
	my $duration = shift;
	my $server = $self->{SERVER};
	my $tmp;

	$self->send_soc("cmd=ctstarttimerasync src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=$duration argc=1 \n");
	$tmp =$self->get_reply($server); 
	$tmp=get_value($tmp,"arg1");
	return($tmp);
}

=pod

=head2 stop_timer()

=over

Stops a timer, returning immediatly.

 $result=$ctport->stop_timer();

=back

=cut

sub stop_timer() {
	my $self = shift;
	my $server = $self->{SERVER};
	my $tmp;

	$self->send_soc("cmd=ctstoptimer src=".$self->{HANDLE}." dst=".$self->{HANDLE}." argc=0 \n");
	$tmp =$self->get_reply($server); 
	$tmp=get_value($tmp,"arg1");
	return($tmp);
}

=pod

=head2 play_tone_async()

=over

Starts a tone playing on this port, returning immediatly.

 $result=$ctport->plat_tone_async($tone);

Valid tones are:

=over

=item *

dial

=item *

dialx

=item *

busy

=item *

ringback

=back

=back

=cut

sub play_tone_async($) {
	my $self = shift;
	my $tone = shift;
	my $server = $self->{SERVER};
	my $tmp;

	$self->send_soc("cmd=ctplaytoneasync src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=$tone argc=1 \n");
	$tmp =$self->get_reply($server); 
	$tmp=get_value($tmp,"arg1");
	return($tmp);
}

=pod

=head2 stop_tone()

=over

Stops an asyncronous tone from playing.

 $result=$ctport->stop_tone();

=back

=cut

sub stop_tone() {
	my $self = shift;
	my $server = $self->{SERVER};
	my $tmp;

	$self->send_soc("cmd=ctstoptone src=".$self->{HANDLE}." dst=".$self->{HANDLE}." argc=0 \n");
	$tmp =$self->get_reply($server); 
	$tmp=get_value($tmp,"arg1");
	return($tmp);
}

=pod

=head2 join()

=over

Joins the audio path of two listed ports together.

 $result=$ctport->join($first_port,$second_port);

=back

=cut

sub join($$) {
	my $self = shift;
	my $port = shift;
	my $port2 = shift;
	my $server = $self->{SERVER};
	my $tmp;

	$self->send_soc("cmd=ctjoin src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=$port arg2=$port2 argc=2 \n");
	$tmp =$self->get_reply($server); 
	$tmp=get_value($tmp,"arg1");
	return($tmp);
}

=pod

=head2 bridge()

=over

Joins/bridges the audio path of this port and a second one together.

 $result=$ctport->($other_port);

=back

=cut

sub bridge($) {
	my $self = shift;
	my $port = shift;
	my $server = $self->{SERVER};
	my $tmp;

	$self->send_soc("cmd=ctbridge src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=$port argc=1 \n");
	$tmp =$self->get_reply($server); 
	$tmp=get_value($tmp,"arg1");
	return($tmp);
}

=pod

=head2 unbridge()

=over

Un-bridges the audio path of this port and another.

 $result=$ctport->unbridge($other_port);

=back

=cut

sub unbridge($) {
	my $self = shift;
	my $port = shift;
	my $server = $self->{SERVER};
	my $tmp;
	if ($port <0){
		return("NOK");
	}

	$self->send_soc("cmd=ctunbridge src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=$port argc=1 \n");
	$tmp =$self->get_reply($server); 
	$tmp=get_value($tmp,"arg1");
	return($tmp);
}

=pod

=head2 start_ring_async()

=over

Starts a station port ringing, returning imediatly.

 $result=$ctport->start_ring_async();

=back

=cut

sub start_ring_async($) {
	my $self = shift;
	my $server = $self->{SERVER};
	my $tmp;

	$self->send_soc("cmd=ctstartringasync src=".$self->{HANDLE}." dst=".$self->{HANDLE}." argc=0 \n");
	$tmp =$self->get_reply($server); 
	$tmp=get_value($tmp,"arg1");
	return($tmp);
}

=pod

=head2 stop_ring()

=over

Stops a station port from ringing.

 $result=$ctport->stop_ring();

=back

=cut

sub stop_ring($) {
	my $self = shift;
	my $server = $self->{SERVER};
	my $tmp;

	$self->send_soc("cmd=ctstopring src=".$self->{HANDLE}." dst=".$self->{HANDLE}." argc=0 \n");
	$tmp =$self->get_reply($server); 
	$tmp=get_value($tmp,"arg1");
	return($tmp);
}

=pod

=head2 start_ring_once_async()

=over

Makes the port ring once, returns imediatly.

 $result=$ctport->start_ring_once_async();

=back

=cut

sub start_ring_once_async() {
	my $self = shift;
	my $server = $self->{SERVER};
	my $tmp;

	$self->send_soc("cmd=ctstartringonceasync src=".$self->{HANDLE}." dst=".$self->{HANDLE}." argc=0 \n");
	$tmp =$self->get_reply($server); 
	$tmp=get_value($tmp,"arg1");
	return($tmp);
}

=pod

=head2 getconf()

=over

Opens a config file and places the results into a hash. The results can be
accessed in this manner:

 $moh_files=$ctport->{CONFIG}->{'files'}{'moh'};

The file format is tag/values pairs seperated by an "=", groups into sections 
by section headers which are quoted in square brackets. eg:

 [files]
 moh="musicone.wav annouce.wav musictwo.wav"

=back

=cut

sub getconf($){
	my $self = shift;
	my $file = shift;
	my ($line,$section,$tag,$value);
	if (!defined $conffile){
		our $conffile = cwd();
		$conffile=$conffile."/".$file;
	}
	open(FILE,"< $conffile") || die "Can't open $conffile : $!\n";
	flock(FILE,LOCK_SH);
	foreach $line (<FILE>){
		chomp $line;
		if ($line =~ /^\s*#/){  # skip comments
			next;
		}
		elsif($line =~ /^\s*\[\w+\]/){ # found section header
			$section=$line;
			$section =~ s/[\[\]]//g;
		}
		elsif($line =~ /^(\w+)=([\W\w]+)/){ # found data
			($tag,$value)=($1,$2);
			#($tag, $value)= split /=/,$line;
			$self->{CONFIG}->{$section}{$tag} = $value;
		}
	}
	flock(FILE,LOCK_UN);
	close FILE;
	return(1);
}

=pod

=head2 save_config()

=over

Compares the current CONFIG hash to the file and then
saves the diferences.

 $ctport->save_config();

=back

=cut

sub save_config(){
	my $self = shift;
	my @newfile;
	if (!defined $conffile){
		$self->logger("Cant save config if it hasnt been loaded");
		return(-1);
	}
	open(FILE,"+< $conffile") || die "Can't open $conffile : $!\n";
	flock(FILE,LOCK_EX);
	foreach $line (<FILE>){
		chomp $line;
		if ($line =~ /^\s*#/){
			push @newfile,$line;
			next;
		}
		elsif($line =~ /^\s*\[\w+\]/){
			$section=$line;
			$section =~ s/[\[\]]//g;
		}
		elsif($line =~ /(\w+)=([\W\w]+)/){
			($tag, $value)= split /=/,$line;
			my $bit =$self->{CONFIG}->{$section}{$tag};
			if ((defined $bit)&&($bit ne $value)){
				$line="$tag=$bit";
			}
		}
		push @newfile,$line;
	}
	seek(FILE,0,0); # rewind
	foreach $line (@newfile){
		print FILE "$line\n";
	}
	flock(FILE,LOCK_UN);
	close FILE;
	# now tell everybody to reload their config!
	foreach $bit (keys %{$self->{CONFIG}{'ports'}}){
		if ($bit ne $self->{HANDLE}){
			$self->send_event($bit,"load_config");
		}
	}
	return(1);

}

=pod

=head2 daemonize()

=over

Daemonizes the script, disconnecting it from the terminal.

 $ctport->deamonize();

=back

=cut

sub daemonize($){
	my $self = shift;
	my $bool = shift;
	if ($bool == 1){
		# make a daemon
		defined(my $pid = fork) or die "Can't fork: $!";
		exit if $pid;
		setsid or die "Can't start a new session: $!";
		chdir '/' or die "Can't chdir to /: $!";
		umask 0;
		open STDIN, '/dev/null' or die "Can't read /dev/null: $!";
		open STDOUT, '/dev/null' or die "Can't write to /dev/null: $!";
		open STDERR, '/dev/null' or die "Can't write to /dev/null: $!";
	}
	$self->{DAEMON}=1;
	return(1);
}

=pod

=head2 openlogger()

=over

Opens a logfile for this port.

 $ctport->openlogger("mylogfile.log");

=back

=cut

sub openlogger($){
	my $self = shift;
	my $logfile = shift;
	if ( defined $self->{CONFIG}{LOGFILE}){
		$logfile=$self->{CONFIG}{LOGFILE};
	}
	else {
		$self->{CONFIG}{LOGFILE}=$logfile;
	}
	my $foo = strftime("%Y%m%d",localtime(time));
	$logfile.=$foo.".log";
	open (our $LOGFILE,">>$logfile") or die("cant open log file =>$logfile<=");
	autoflush $LOGFILE 1;
}

=pod

=head2 logger()

=over

Logs to the log file opened with openlogger.

 $ctport->logger("New call from $cli");

=back

=cut

sub logger($){
	my $self = shift;
	my $text = shift;
	my $foo = strftime("%Y/%m/%d-%H:%M:%S",localtime(time));
	if ($self->{DAEMON}){
		print $LOGFILE "$foo [".$port."] $text\n";
	}
	else {
		print STDERR "$foo [".$port."] $text\n";
	}
}

=pod

=head2 closelogger()

=over

Closes the logfile for this port.

 $ctport->closelogger();

=back

=cut

sub closelogger($){
	my $self = shift;
	close (our $LOGFILE);
}

=pod

=head2 gruntoff()

=over

Turns off grunt detection. Stops "grunt" events.

 $ctport->gruntoff;

=back

=cut

sub gruntoff(){
	my $self = shift;
	my $server = $self->{SERVER};
	my $tmp;

	$self->send_soc("cmd=ctgruntoff src=".$self->{HANDLE}." dst=".$self->{HANDLE}." argc=0 \n");
	$tmp =$self->get_reply($server); 
}

=pod

=head2 grunton()

=over

Turns on grunt detection. This will produce "grunt" events. Usefull for
detecting lack of grunts (ie silence).

 $ctport->grunton;

=back

=cut

sub grunton(){
	my $self = shift;
	my $server = $self->{SERVER};
	my $tmp;

	$self->send_soc("cmd=ctgrunton src=".$self->{HANDLE}." dst=".$self->{HANDLE}." argc=0 \n");
	$tmp =$self->get_reply($server); 
}

=pod

=head2 ignore_dtmf_on()

=over


Stops play files from being interupted by DTMF.

 $ctport->ignore_dtmf_on;

=back

=cut

sub ignore_dtmf_on(){
	my $self = shift;
	$self->{IGNORE_DTMF} = 1;
}


=pod

=head2 ignore_dtmf_off()

=over


Allows play files to be interupted by DTMF.

 $ctport->ignore_dtmf_off;

=back

=cut

sub ignore_dtmf_off(){
	my $self = shift;
	$self->{IGNORE_DTMF} = 0;
}


=pod

=head2 setscript()

=over

Sets the script information available from the management thread.

 $ctport->setscript("MainMenu");

=back

=cut

sub setscript($){
	my $self = shift;
	my $text = shift;
	my $server = $self->{SERVER};
	my $tmp;

	$self->send_soc("cmd=ctsetscript src=".$self->{HANDLE}." dst=".$self->{HANDLE}." arg1=".$text." argc=1 \n");
	$tmp =$self->get_reply($server); 
}

sub get_value($$){
	my $text = shift;
	my $search= shift;
	# This regex can be broken onto three parts, the locator,
	# the data, the end marker. The locator is "$search=", the data
	# is "([\*\#\w\s].*?)" which translates to "any white space, word
	# characters, * and # in groups of 1 or more." The end marker is
	# "(\s\w{3,4}=.*)" which translates "a white space, followed by 3
	# or 4 word charcters, then and equals sign and anything else.
	#
	if ($text=~/$search=([\*\#\w\s].*?)(\s\w{3,4}=.*)*?$/){
		my $bit=$1;
		if ($bit =~/^ /){
			return("");
		}
		return($1);
	}
	else {
		return("");
	}
}

sub get_reply($){
	my $self=shift;
	my $server=shift;
	my $tmp="";
	my $bit="";
	while (read $server,$bit,1){
		if($bit eq "\n"){
			#$self->logger("get_reply:[$tmp]");
			return($tmp);
		}
		if ($bit ne chr(0)){
			$tmp.=$bit;
		}
	}
}

sub send_soc($){
	my $self = shift;
	my $srv = $self->{SERVER};
	my $text=shift;
	print $srv "$text";
	chomp($text);
	#$self->logger("send_soc:[$text]");
}

1;

